"""
Testsuite for ProfilePlot and PhasePlot



"""

#-----------------------------------------------------------------------------
# Copyright (c) 2013, yt Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
#-----------------------------------------------------------------------------
import os
import tempfile
import shutil
import unittest
import yt
from yt.data_objects.profiles import create_profile
from yt.testing import \
    fake_random_ds, \
    assert_array_almost_equal, \
    requires_file, \
    assert_fname
from yt.visualization.profile_plotter import \
    ProfilePlot, PhasePlot
from yt.visualization.tests.test_plotwindow import \
    TEST_FLNMS
from yt.utilities.answer_testing.framework import \
    PhasePlotAttributeTest, \
    requires_ds, \
    data_dir_load

ATTR_ARGS = {"annotate_text": [(((5e-29, 5e7), "Hello YT"), {}), 
                               (((5e-29, 5e7), "Hello YT"), {'color':'b'})],
             
             "set_title": [(('cell_mass', 'A phase plot.'), {})],
             "set_log": [(('cell_mass', False), {})],
             "set_unit": [(('cell_mass', 'Msun'), {})],
             "set_xlim": [((1e-27, 1e-24), {})],
             "set_ylim": [((1e2, 1e6), {})]}


g30 = "IsolatedGalaxy/galaxy0030/galaxy0030"

@requires_ds(g30, big_data=True)
def test_phase_plot_attributes():
    '''

    This iterates over the all the plot modification functions in 
    ATTR_ARGS. Each time, it compares the images produced by 
    PhasePlot to the gold standard.
    

    '''

    x_field = 'density'
    y_field = 'temperature'
    z_field = 'cell_mass'
    decimals = 12
    ds = data_dir_load(g30)
    for attr_name in ATTR_ARGS.keys():
        for args in ATTR_ARGS[attr_name]:
            test = PhasePlotAttributeTest(ds, x_field, y_field, z_field, 
                                          attr_name, args, decimals)
            test_phase_plot_attributes.__name__ = test.description
            yield test

class TestProfilePlotSave(unittest.TestCase):

    def setUp(self):
        self.tmpdir = tempfile.mkdtemp()
        self.curdir = os.getcwd()
        os.chdir(self.tmpdir)

    def tearDown(self):
        os.chdir(self.curdir)
        shutil.rmtree(self.tmpdir)

    def test_profile_plot(self):
        fields = ('density', 'temperature', 'velocity_x', 'velocity_y',
                  'velocity_z')
        units = ('g/cm**3', 'K', 'cm/s', 'cm/s', 'cm/s')
        test_ds = fake_random_ds(16, fields=fields, units=units)
        regions = [test_ds.region([0.5]*3, [0.4]*3, [0.6]*3), test_ds.all_data()]
        pr_fields = [('density', 'temperature'), ('density', 'velocity_x'),
                     ('temperature', 'cell_mass'), ('density', 'radius'),
                     ('velocity_magnitude', 'cell_mass')]
        profiles = []
        for reg in regions:
            for x_field, y_field in pr_fields:
                profiles.append(ProfilePlot(reg, x_field, y_field))
                profiles.append(ProfilePlot(reg, x_field, y_field,
                                            fractional=True, accumulation=True))
                p1d = create_profile(reg, x_field, y_field)
                profiles.append(ProfilePlot.from_profiles(p1d))
        p1 = create_profile(test_ds.all_data(), 'density', 'temperature')
        p2 = create_profile(test_ds.all_data(), 'density', 'velocity_x')
        profiles.append(ProfilePlot.from_profiles(
            [p1, p2], labels=['temperature', 'velocity']))
        profiles[0]._repr_html_()
        for p in profiles:
            for fname in TEST_FLNMS:
                assert_fname(p.save(fname)[0])

    def test_phase_plot(self):
        fields = ('density', 'temperature', 'velocity_x', 'velocity_y',
                  'velocity_z')
        units = ('g/cm**3', 'K', 'cm/s', 'cm/s', 'cm/s')
        test_ds = fake_random_ds(16, fields=fields, units=units)
        regions = [test_ds.region([0.5]*3, [0.4]*3, [0.6]*3), test_ds.all_data()]
        phases = []
        ph_fields = [('density', 'temperature', 'cell_mass'),
                     ('density', 'velocity_x', 'cell_mass'),
                     ('radius', 'temperature', 'velocity_magnitude')]
        for reg in regions:
            for x_field, y_field, z_field in ph_fields:
                # set n_bins to [16, 16] since matplotlib's postscript
                # renderer is slow when it has to write a lot of polygons
                phases.append(PhasePlot(reg, x_field, y_field, z_field,
                                        x_bins=16, y_bins=16))
                phases.append(PhasePlot(reg, x_field, y_field, z_field,
                                        fractional=True, accumulation=True,
                                        x_bins=16, y_bins=16))
                p2d = create_profile(reg, [x_field, y_field], z_field,
                                     n_bins=[16, 16])
                phases.append(PhasePlot.from_profile(p2d))
        pp = PhasePlot(test_ds.all_data(), 'density', 'temperature', 'cell_mass')
        pp.set_xlim(0.3, 0.8)
        pp.set_ylim(0.4, 0.6)
        pp._setup_plots()
        xlim = pp.plots['cell_mass'].axes.get_xlim()
        ylim = pp.plots['cell_mass'].axes.get_ylim()
        assert_array_almost_equal(xlim, (0.3, 0.8))
        assert_array_almost_equal(ylim, (0.4, 0.6))
        phases.append(pp)
        phases[0]._repr_html_()
        for p in phases:
            for fname in TEST_FLNMS:
                assert_fname(p.save(fname)[0])

ETC46 = "enzo_tiny_cosmology/DD0046/DD0046"

@requires_file(ETC46)
def test_profile_plot_multiple_field_multiple_plot():
    ds = yt.load(ETC46)
    sphere = ds.sphere("max", (1.0, "Mpc"))
    profiles = []
    profiles.append(yt.create_profile(
        sphere, ["radius"],
        fields=["density"],n_bins=32))
    profiles.append(yt.create_profile(
        sphere, ["radius"],
        fields=["density"],n_bins=64))
    profiles.append(yt.create_profile(
        sphere, ["radius"],
        fields=["dark_matter_density"],n_bins=64))

    plot = yt.ProfilePlot.from_profiles(profiles)
    with tempfile.NamedTemporaryFile(suffix='png') as f:
        plot.save(name=f.name)

@requires_file(ETC46)
def test_set_units():
    ds = yt.load(ETC46)
    sp = ds.sphere("max", (1.0, "Mpc"))
    p1 = yt.ProfilePlot(sp, "radius", ("enzo", "Density"))
    p2 = yt.PhasePlot(sp, ("enzo", "Density"), ("enzo", "Temperature"), "cell_mass")
    # make sure we can set the units using the tuple without erroring out
    p1.set_unit(("enzo", "Density"), "Msun/kpc**3")
    p2.set_unit(("enzo", "Temperature"), "R")


"""
Domain context base class

Currently we largely apply this to the fields that get loaded.  Presumably
different analysis operations could be the subject of this type of examination
as well.
"""

#-----------------------------------------------------------------------------
# Copyright (c) 2013, yt Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
#-----------------------------------------------------------------------------

domain_context_registry = {}

class DomainContext(object):
    class __metaclass__(type):
        def __init__(cls, name, b, d):
            type.__init__(cls, name, b, d)
            domain_context_registry[name] = cls

    _known_fluid_fields     =  ()
    _known_particle_fields  =  ()

    def __init__(self, ds):
        self.ds = ds




"""
Definitions for geographic coordinate systems




"""

#-----------------------------------------------------------------------------
# Copyright (c) 2013, yt Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
#-----------------------------------------------------------------------------

import numpy as np
from .coordinate_handler import \
    CoordinateHandler, \
    _unknown_coord, \
    _get_coord_fields
from yt.utilities.lib.pixelization_routines import \
    pixelize_cylinder, pixelize_aitoff

class GeographicCoordinateHandler(CoordinateHandler):
    radial_axis = "altitude"
    name = "geographic"

    def __init__(self, ds, ordering = None):
        if not ordering:
            ordering = ('latitude', 'longitude', self.radial_axis)
        super(GeographicCoordinateHandler, self).__init__(ds, ordering)
        self.image_units = {}
        self.image_units[self.axis_id['latitude']] = (None, None)
        self.image_units[self.axis_id['longitude']] = (None, None)
        self.image_units[self.axis_id[self.radial_axis]] = ('deg', 'deg')

    def setup_fields(self, registry):
        # return the fields for r, z, theta
        registry.add_field(("index", "dx"), sampling_type="cell",  function=_unknown_coord)
        registry.add_field(("index", "dy"), sampling_type="cell",  function=_unknown_coord)
        registry.add_field(("index", "dz"), sampling_type="cell",  function=_unknown_coord)
        registry.add_field(("index", "x"), sampling_type="cell",  function=_unknown_coord)
        registry.add_field(("index", "y"), sampling_type="cell",  function=_unknown_coord)
        registry.add_field(("index", "z"), sampling_type="cell",  function=_unknown_coord)
        f1, f2 = _get_coord_fields(self.axis_id['latitude'], "")
        registry.add_field(("index", "dlatitude"), sampling_type="cell",  function = f1,
                           display_field = False,
                           units = "")
        registry.add_field(("index", "latitude"), sampling_type="cell",  function = f2,
                           display_field = False,
                           units = "")

        f1, f2 = _get_coord_fields(self.axis_id['longitude'], "")
        registry.add_field(("index", "dlongitude"), sampling_type="cell",  function = f1,
                           display_field = False,
                           units = "")
        registry.add_field(("index", "longitude"), sampling_type="cell",  function = f2,
                           display_field = False,
                           units = "")

        f1, f2 = _get_coord_fields(self.axis_id[self.radial_axis])
        registry.add_field(("index", "d%s" % (self.radial_axis,)), sampling_type="cell",  function = f1,
                           display_field = False,
                           units = "code_length")
        registry.add_field(("index", self.radial_axis), sampling_type="cell",  function = f2,
                           display_field = False,
                           units = "code_length")

        def _SphericalVolume(field, data):
            # We can use the transformed coordinates here.
            # Here we compute the spherical volume element exactly
            r = data["index", "r"]
            dr = data["index", "dr"]
            theta = data["index", "theta"]
            dtheta = data["index", "dtheta"]
            vol = ((r+0.5*dr)**3-(r-0.5*dr)**3) / 3.0
            vol *= np.cos(theta-0.5*dtheta)-np.cos(theta+0.5*dtheta)
            vol *= data["index", "dphi"]
            return vol
        registry.add_field(("index", "cell_volume"), sampling_type="cell",
                 function=_SphericalVolume,
                 units = "code_length**3")

        def _path_radial_axis(field, data):
            return data["index", "d%s" % self.radial_axis]
        registry.add_field(("index", "path_element_%s" % self.radial_axis), sampling_type="cell", 
                 function = _path_radial_axis,
                 units = "code_length")
        def _path_latitude(field, data):
            # We use r here explicitly
            return data["index", "r"] * \
                data["index", "dlatitude"] * np.pi/180.0
        registry.add_field(("index", "path_element_latitude"), sampling_type="cell", 
                 function = _path_latitude,
                 units = "code_length")
        def _path_longitude(field, data):
            # We use r here explicitly
            return data["index", "r"] \
                    * data["index", "dlongitude"] * np.pi/180.0 \
                    * np.sin((data["index", "latitude"] + 90.0) * np.pi/180.0)
        registry.add_field(("index", "path_element_longitude"), sampling_type="cell", 
                 function = _path_longitude,
                 units = "code_length")

        def _latitude_to_theta(field, data):
            # latitude runs from -90 to 90
            return (data["latitude"] + 90) * np.pi/180.0
        registry.add_field(("index", "theta"), sampling_type="cell", 
                 function = _latitude_to_theta,
                 units = "")
        def _dlatitude_to_dtheta(field, data):
            return data["dlatitude"] * np.pi/180.0
        registry.add_field(("index", "dtheta"), sampling_type="cell", 
                 function = _dlatitude_to_dtheta,
                 units = "")

        def _longitude_to_phi(field, data):
            # longitude runs from -180 to 180
            return (data["longitude"] + 180) * np.pi/180.0
        registry.add_field(("index", "phi"), sampling_type="cell", 
                 function = _longitude_to_phi,
                 units = "")
        def _dlongitude_to_dphi(field, data):
            return data["dlongitude"] * np.pi/180.0
        registry.add_field(("index", "dphi"), sampling_type="cell", 
                 function = _dlongitude_to_dphi,
                 units = "")

        self._setup_radial_fields(registry)

    def _setup_radial_fields(self, registry):
        # This stays here because we don't want to risk the field detector not
        # properly getting the data_source, etc.
        def _altitude_to_radius(field, data):
            surface_height = data.get_field_parameter("surface_height")
            if surface_height is None:
                if hasattr(data.ds, "surface_height"):
                    surface_height = data.ds.surface_height
                else:
                    surface_height = data.ds.quan(0.0, "code_length")
            return data["altitude"] + surface_height
        registry.add_field(("index", "r"), sampling_type="cell", 
                 function=_altitude_to_radius,
                 units = "code_length")
        registry.alias(("index", "dr"), ("index", "daltitude"))

    def _retrieve_radial_offset(self, data_source = None):
        # This returns the factor by which the radial field is multiplied and
        # the scalar its offset by.  Typically the "factor" will *only* be
        # either 1.0 or -1.0.  The order will be factor * r + offset.
        # Altitude is the radius from the central zone minus the radius of the
        # surface.  Depth to radius is negative value of depth plus the
        # outermost radius.
        surface_height = None
        if data_source is not None:
            surface_height = data_source.get_field_parameter("surface_height")
        if surface_height is None:
            if hasattr(self.ds, "surface_height"):
                surface_height = self.ds.surface_height
            else:
                surface_height = self.ds.quan(0.0, "code_length")
        return surface_height, 1.0

    def pixelize(self, dimension, data_source, field, bounds, size,
                 antialias = True, periodic = True):
        if self.axis_name[dimension] in ('latitude', 'longitude'):
            return self._cyl_pixelize(data_source, field, bounds, size,
                                          antialias, dimension)
        elif self.axis_name[dimension] == self.radial_axis:
            return self._ortho_pixelize(data_source, field, bounds, size,
                                        antialias, dimension, periodic)
        else:
            raise NotImplementedError

    def _ortho_pixelize(self, data_source, field, bounds, size, antialias,
                        dim, periodic):
        # For axis=2, x axis will be latitude, y axis will be longitude
        px = (data_source["px"].d + 90) * np.pi/180
        pdx = data_source["pdx"].d * np.pi/180
        py = (data_source["py"].d + 180) * np.pi/180
        pdy = data_source["pdy"].d * np.pi/180
        # First one in needs to be the equivalent of "theta", which is
        # longitude
        buff = pixelize_aitoff(py, pdy, px, pdx,
                               size, data_source[field], None,
                               None).transpose()
        return buff

    def _cyl_pixelize(self, data_source, field, bounds, size, antialias,
                      dimension):
        offset, factor = self._retrieve_radial_offset(data_source)
        r = factor * data_source['py'] + offset
        # Because of the axis-ordering, dimensions 0 and 1 both have r as py
        # and the angular coordinate as px.  But we need to figure out how to
        # convert our coordinate back to an actual angle, based on which
        # dimension we're in.
        pdx = data_source['pdx'].d * np.pi/180
        if self.axis_name[self.x_axis[dimension]] == 'latitude':
            px = (data_source['px'].d + 90) * np.pi/180
            do_transpose = True
        elif self.axis_name[self.x_axis[dimension]] == 'longitude':
            px = (data_source['px'].d + 180) * np.pi/180
            do_transpose = False
        else:
            # We should never get here!
            raise NotImplementedError
        buff = np.zeros((size[1], size[0]), dtype="f8")
        pixelize_cylinder(buff, r, data_source['pdy'],
                          px, pdx, data_source[field], bounds)
        if do_transpose:
            buff = buff.transpose()
        return buff

    def convert_from_cartesian(self, coord):
        raise NotImplementedError

    def convert_to_cartesian(self, coord):
        offset, factor = self._retrieve_radial_offset()
        if isinstance(coord, np.ndarray) and len(coord.shape) > 1:
            rad = self.axis_id[self.radial_axis]
            lon = self.axis_id['longitude']
            lat = self.axis_id['latitude']
            r = factor * coord[:,rad] + offset
            theta = coord[:,lon] * np.pi/180
            phi = coord[:,lat] * np.pi/180
            nc = np.zeros_like(coord)
            # r, theta, phi
            nc[:,lat] = np.cos(phi) * np.sin(theta)*r
            nc[:,lon] = np.sin(phi) * np.sin(theta)*r
            nc[:,rad] = np.cos(theta) * r
        else:
            a, b, c = coord
            theta = b * np.pi/180
            phi = a * np.pi/180
            r = factor * c + offset
            nc = (np.cos(phi) * np.sin(theta)*r,
                  np.sin(phi) * np.sin(theta)*r,
                  np.cos(theta) * r)
        return nc

    def convert_to_cylindrical(self, coord):
        raise NotImplementedError

    def convert_from_cylindrical(self, coord):
        raise NotImplementedError

    def convert_to_spherical(self, coord):
        raise NotImplementedError

    def convert_from_spherical(self, coord):
        raise NotImplementedError

    _image_axis_name = None
    @property
    def image_axis_name(self):    
        if self._image_axis_name is not None:
            return self._image_axis_name
        # This is the x and y axes labels that get displayed.  For
        # non-Cartesian coordinates, we usually want to override these for
        # Cartesian coordinates, since we transform them.
        rv = {self.axis_id['latitude']:
                 ('x / \\sin(\mathrm{latitude})',
                  'y / \\sin(\mathrm{latitude})'),
              self.axis_id['longitude']:
                 ('R', 'z'),
              self.axis_id[self.radial_axis]:
                 ('longitude', 'latitude')}
        for i in list(rv.keys()):
            rv[self.axis_name[i]] = rv[i]
            rv[self.axis_name[i].capitalize()] = rv[i]
        self._image_axis_name = rv
        return rv

    _x_pairs = (('latitude', 'longitude'),
                ('longitude', 'latitude'),
                ('altitude', 'latitude'))

    _y_pairs = (('latitude', 'altitude'),
                ('longitude', 'altitude'),
                ('altitude', 'longitude'))

    @property
    def period(self):
        return self.ds.domain_width

    def sanitize_center(self, center, axis):
        center, display_center = super(
            GeographicCoordinateHandler, self).sanitize_center(center, axis)
        name = self.axis_name[axis]
        if name == self.radial_axis:
            display_center = center
        elif name == 'latitude':
            display_center = (0.0 * display_center[0],
                              0.0 * display_center[1],
                              0.0 * display_center[2])
        elif name == 'longitude':
            ri = self.axis_id[self.radial_axis]
            c = (self.ds.domain_right_edge[ri] +
                 self.ds.domain_left_edge[ri])/2.0
            display_center = [0.0 * display_center[0], 
                              0.0 * display_center[1],
                              0.0 * display_center[2]]
            display_center[self.axis_id['latitude']] = c
        return center, display_center

    def sanitize_width(self, axis, width, depth):
        name = self.axis_name[axis]
        if width is not None:
            width = super(GeographicCoordinateHandler, self).sanitize_width(
              axis, width, depth)
        elif name == self.radial_axis:
            rax = self.radial_axis
            width = [self.ds.domain_width[self.x_axis[rax]],
                     self.ds.domain_width[self.y_axis[rax]]]
        elif name == 'latitude':
            ri = self.axis_id[self.radial_axis]
            # Remember, in spherical coordinates when we cut in theta,
            # we create a conic section
            width = [2.0*self.ds.domain_width[ri],
                     2.0*self.ds.domain_width[ri]]
        elif name == 'longitude':
            ri = self.axis_id[self.radial_axis]
            width = [self.ds.domain_width[ri],
                     2.0*self.ds.domain_width[ri]]
        return width

class InternalGeographicCoordinateHandler(GeographicCoordinateHandler):
    radial_axis = "depth"
    name = "internal_geographic"
    
    def _setup_radial_fields(self, registry):
        # Altitude is the radius from the central zone minus the radius of the
        # surface.
        def _depth_to_radius(field, data):
            outer_radius = data.get_field_parameter("outer_radius")
            if outer_radius is None:
                if hasattr(data.ds, "outer_radius"):
                    outer_radius = data.ds.outer_radius
                else:
                    # Otherwise, we assume that the depth goes to full depth,
                    # so we can look at the domain right edge in depth.
                    rax = self.axis_id[self.radial_axis]
                    outer_radius = data.ds.domain_right_edge[rax]
            return -1.0 * data["depth"] + outer_radius
        registry.add_field(("index", "r"), sampling_type="cell", 
                 function=_depth_to_radius,
                 units = "code_length")
        registry.alias(("index", "dr"), ("index", "ddepth"))

    def _retrieve_radial_offset(self, data_source = None):
        # Depth means switching sign and adding to full radius
        outer_radius = None
        if data_source is not None:
            outer_radius = data_source.get_field_parameter("outer_radius")
        if outer_radius is None:
            if hasattr(self.ds, "outer_radius"):
                outer_radius = self.ds.outer_radius
            else:
                # Otherwise, we assume that the depth goes to full depth,
                # so we can look at the domain right edge in depth.
                rax = self.axis_id[self.radial_axis]
                outer_radius = self.ds.domain_right_edge[rax]
        return outer_radius, -1.0

    _x_pairs = (('latitude', 'longitude'),
                ('longitude', 'latitude'),
                ('depth', 'latitude'))

    _y_pairs = (('latitude', 'depth'),
                ('longitude', 'depth'),
                ('depth', 'longitude'))

    def sanitize_center(self, center, axis):
        center, display_center = super(
            GeographicCoordinateHandler, self).sanitize_center(center, axis)
        name = self.axis_name[axis]
        if name == self.radial_axis:
            display_center = center
        elif name == 'latitude':
            display_center = (0.0 * display_center[0],
                              0.0 * display_center[1],
                              0.0 * display_center[2])
        elif name == 'longitude':
            ri = self.axis_id[self.radial_axis]
            offset, factor = self._retrieve_radial_offset()
            outermost = factor * self.ds.domain_left_edge[ri] + offset
            display_center = [0.0 * display_center[0], 
                              0.0 * display_center[1],
                              0.0 * display_center[2]]
            display_center[self.axis_id['latitude']] = outermost/2.0
        return center, display_center

    def sanitize_width(self, axis, width, depth):
        name = self.axis_name[axis]
        if width is not None:
            width = super(GeographicCoordinateHandler, self).sanitize_width(
              axis, width, depth)
        elif name == self.radial_axis:
            rax = self.radial_axis
            width = [self.ds.domain_width[self.y_axis[rax]],
                     self.ds.domain_width[self.x_axis[rax]]]
        elif name == 'latitude':
            ri = self.axis_id[self.radial_axis]
            # Remember, in spherical coordinates when we cut in theta,
            # we create a conic section
            offset, factor = self._retrieve_radial_offset()
            outermost = factor * self.ds.domain_left_edge[ri] + offset
            width = [2.0*outermost, 2.0*outermost]
        elif name == 'longitude':
            ri = self.axis_id[self.radial_axis]
            offset, factor = self._retrieve_radial_offset()
            outermost = factor * self.ds.domain_left_edge[ri] + offset
            width = [outermost, 2.0*outermost]
        return width




import os
import shutil
import tempfile

import numpy as np

from unittest import TestCase

from yt.testing import \
    fake_random_ds, \
    assert_almost_equal, \
    assert_equal, \
    requires_file

from yt.convenience import \
    load

def setup():
    from yt.config import ytcfg
    ytcfg["yt","__withintesting"] = "True"

def test_flux_calculation():
    ds = fake_random_ds(64, nprocs = 4)
    dd = ds.all_data()
    surf = ds.surface(dd, "x", 0.51)
    assert_equal(surf["x"], 0.51)
    flux = surf.calculate_flux("ones", "zeros", "zeros", "ones")
    assert_almost_equal(flux.value, 1.0, 12)
    assert_equal(str(flux.units), 'cm**2')

def test_sampling():
    ds = fake_random_ds(64, nprocs = 4)
    dd = ds.all_data()
    for i, ax in enumerate('xyz'):
        surf = ds.surface(dd, ax, 0.51)
        surf.get_data(ax, "vertex")
        assert_equal(surf.vertex_samples[ax], surf.vertices[i,:])
        assert_equal(str(surf.vertices.units), 'code_length')
        dens = surf['density']
        vert_shape = surf.vertices.shape
        assert_equal(dens.shape[0], vert_shape[1]//vert_shape[0])
        assert_equal(str(dens.units), 'g/cm**3')

ISOGAL = 'IsolatedGalaxy/galaxy0030/galaxy0030'

class ExporterTests(TestCase):

    def setUp(self):
        self.curdir = os.getcwd()
        self.tmpdir = tempfile.mkdtemp()
        os.chdir(self.tmpdir)

    def tearDown(self):
        os.chdir(self.curdir)
        shutil.rmtree(self.tmpdir)

    def test_export_ply(self):
        ds = fake_random_ds(64, nprocs = 4)
        dd = ds.all_data()
        surf = ds.surface(dd, 'x', 0.51)
        surf.export_ply('my_ply.ply', bounds=[(0, 1), (0, 1), (0, 1)])
        assert os.path.exists('my_ply.ply')

    @requires_file(ISOGAL)
    def test_export_obj(self):
        ds = load(ISOGAL)
        sp = ds.sphere("max", (10, "kpc"))
        trans = 1.0
        distf = 3.1e18*1e3 # distances into kpc
        surf = ds.surface(sp, "density", 5e-27)
        surf.export_obj("my_galaxy", transparency=trans, dist_fac = distf)
        assert os.path.exists('my_galaxy.obj')
        assert os.path.exists('my_galaxy.mtl')

        mi, ma = sp.quantities.extrema('temperature')
        rhos = [1e-24, 1e-25]
        trans = [0.5, 1.0]
        for i, r in enumerate(rhos):
            surf = ds.surface(sp,'density',r)
            surf.export_obj("my_galaxy_color".format(i),
                            transparency=trans[i],
                            color_field='temperature', dist_fac = distf,
                            plot_index = i, color_field_max = ma,
                            color_field_min = mi)

        assert os.path.exists('my_galaxy_color.obj')
        assert os.path.exists('my_galaxy_color.mtl')

        def _Emissivity(field, data):
            return (data['density']*data['density'] *
                    np.sqrt(data['temperature']))
        ds.add_field("emissivity", sampling_type='cell', function=_Emissivity,
                     units=r"g**2*sqrt(K)/cm**6")
        for i, r in enumerate(rhos):
            surf = ds.surface(sp,'density',r)
            surf.export_obj("my_galaxy_emis".format(i),
                            transparency=trans[i],
                            color_field='temperature',
                            emit_field='emissivity',
                            dist_fac = distf, plot_index = i)

        assert os.path.exists('my_galaxy_emis.obj')
        assert os.path.exists('my_galaxy_emis.mtl')

@requires_file(ISOGAL)
def test_correct_output_unit():
    # see issue #1368
    ds = load(ISOGAL)
    x = y = z = .5
    sp1 = ds.sphere((x,y,z), (300, 'kpc'))
    Nmax = sp1.max('HI_Density')
    sur = ds.surface(sp1,"HI_Density", .5*Nmax)
    sur['x'][0]

@requires_file(ISOGAL)
def test_radius_surface():
    # see #1407
    ds = load(ISOGAL)
    reg = ds.all_data()
    sp = ds.sphere(ds.domain_center, (0.5, 'code_length'))
    for obj in [reg, sp]:
        for rad in [0.05, .1, .4]:
            surface = ds.surface(obj, 'radius', (rad, 'code_length'))
            assert_almost_equal(
                surface.surface_area.v, 4*np.pi*rad**2, decimal=2)
            verts = surface.vertices
            for i in range(3):
                assert_almost_equal(
                    verts[i, :].min().v, 0.5-rad, decimal=2)
                assert_almost_equal(
                    verts[i, :].max().v, 0.5+rad, decimal=2)



"""
Make a fake octree, deposit particle at every leaf




"""

#-----------------------------------------------------------------------------
# Copyright (c) 2013, yt Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
#-----------------------------------------------------------------------------

from libc.stdlib cimport malloc, free, rand, RAND_MAX
cimport numpy as np
from oct_visitors cimport cind
import numpy as np
cimport cython

from oct_container cimport Oct, RAMSESOctreeContainer

# Create a balanced octree by a random walk that recursively
# subdivides
def create_fake_octree(RAMSESOctreeContainer oct_handler,
                       long max_noct,
                       long max_level,
                       np.ndarray[np.int32_t, ndim=1] ndd,
                       np.ndarray[np.float64_t, ndim=1] dle,
                       np.ndarray[np.float64_t, ndim=1] dre,
                       float fsubdivide):
    cdef int[3] dd #hold the octant index
    cdef int[3] ind #hold the octant index
    cdef long i
    cdef long cur_leaf = 0
    cdef np.ndarray[np.uint8_t, ndim=2] mask
    for i in range(3):
        ind[i] = 0
        dd[i] = ndd[i]
    oct_handler.allocate_domains([max_noct])
    parent = oct_handler.next_root(1, ind)
    parent.domain = 1
    cur_leaf = 8 #we've added one parent...
    mask = np.ones((max_noct,8),dtype='uint8')
    while oct_handler.domains[0].n_assigned < max_noct:
        print "root: nocts ", oct_handler.domains[0].n_assigned
        cur_leaf = subdivide(oct_handler, parent, ind, dd, cur_leaf, 0,
                             max_noct, max_level, fsubdivide, mask)
    return cur_leaf


cdef long subdivide(RAMSESOctreeContainer oct_handler,
                    Oct *parent,
                    int ind[3], int dd[3],
                    long cur_leaf, long cur_level,
                    long max_noct, long max_level, float fsubdivide,
                    np.ndarray[np.uint8_t, ndim=2] mask):
    print "child", parent.file_ind, ind[0], ind[1], ind[2], cur_leaf, cur_level
    cdef int ddr[3]
    cdef int ii
    cdef long i
    cdef float rf #random float from 0-1
    if cur_level >= max_level:
        return cur_leaf
    if oct_handler.domains[0].n_assigned >= max_noct:
        return cur_leaf
    for i in range(3):
        ind[i] = <int> ((rand() * 1.0 / RAND_MAX) * dd[i])
        ddr[i] = 2
    rf = rand() * 1.0 / RAND_MAX
    if rf > fsubdivide:
        ii = cind(ind[0], ind[1], ind[2])
        if parent.children[ii] == NULL:
            cur_leaf += 7
        oct = oct_handler.next_child(1, ind, parent)
        oct.domain = 1
        cur_leaf = subdivide(oct_handler, oct, ind, ddr, cur_leaf,
                             cur_level + 1, max_noct, max_level,
                             fsubdivide, mask)
    return cur_leaf



"""
HTTPStream data-file handling function




"""
from __future__ import print_function

#-----------------------------------------------------------------------------
# Copyright (c) 2014, yt Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
#-----------------------------------------------------------------------------

import numpy as np

from yt.funcs import \
    get_requests, \
    mylog
from yt.utilities.io_handler import \
    BaseIOHandler
from yt.utilities.lib.geometry_utils import \
    compute_morton

class IOHandlerHTTPStream(BaseIOHandler):
    _dataset_type = "http_particle_stream"
    _vector_fields = ("Coordinates", "Velocity", "Velocities")

    def __init__(self, ds):
        if get_requests() is None:
            raise ImportError(
                "This functionality depends on the requests package")
        self._url = ds.base_url
        # This should eventually manage the IO and cache it
        self.total_bytes = 0
        super(IOHandlerHTTPStream, self).__init__(ds)

    def _open_stream(self, data_file, field):
        # This does not actually stream yet!
        ftype, fname = field
        s = "%s/%s/%s/%s" % (self._url,
            data_file.file_id, ftype, fname)
        mylog.info("Loading URL %s", s)
        requests = get_requests()
        resp = requests.get(s)
        if resp.status_code != 200:
            raise RuntimeError
        self.total_bytes += len(resp.content)
        return resp.content

    def _identify_fields(self, data_file):
        f = []
        for ftype, fname in self.ds.parameters["field_list"]:
            f.append((str(ftype), str(fname)))
        return f, {}

    def _read_particle_coords(self, chunks, ptf):
        chunks = list(chunks)
        data_files = set([])
        for chunk in chunks:
            for obj in chunk.objs:
                data_files.update(obj.data_files)
        for data_file in sorted(data_files):
            for ptype in ptf:
                s = self._open_stream(data_file, (ptype, "Coordinates"))
                c = np.frombuffer(s, dtype="float64")
                c.shape = (c.shape[0]/3.0, 3)
                yield ptype, (c[:,0], c[:,1], c[:,2])

    def _read_particle_fields(self, chunks, ptf, selector):
        # Now we have all the sizes, and we can allocate
        data_files = set([])
        for chunk in chunks:
            for obj in chunk.objs:
                data_files.update(obj.data_files)
        for data_file in sorted(data_files):
            for ptype, field_list in sorted(ptf.items()):
                s = self._open_stream(data_file, (ptype, "Coordinates"))
                c = np.frombuffer(s, dtype="float64")
                c.shape = (c.shape[0]/3.0, 3)
                mask = selector.select_points(
                            c[:,0], c[:,1], c[:,2], 0.0)
                del c
                if mask is None: continue
                for field in field_list:
                    s = self._open_stream(data_file, (ptype, field))
                    c = np.frombuffer(s, dtype="float64")
                    if field in self._vector_fields:
                        c.shape = (c.shape[0]/3.0, 3)
                    data = c[mask, ...]
                    yield (ptype, field), data

    def _initialize_index(self, data_file, regions):
        header = self.ds.parameters
        ptypes = header["particle_count"][data_file.file_id].keys()
        pcount = sum(header["particle_count"][data_file.file_id].values())
        morton = np.empty(pcount, dtype='uint64')
        ind = 0
        for ptype in ptypes:
            s = self._open_stream(data_file, (ptype, "Coordinates"))
            c = np.frombuffer(s, dtype="float64")
            c.shape = (c.shape[0]/3.0, 3)
            regions.add_data_file(c, data_file.file_id,
                                  data_file.ds.filter_bbox)
            morton[ind:ind+c.shape[0]] = compute_morton(
                c[:,0], c[:,1], c[:,2],
                data_file.ds.domain_left_edge,
                data_file.ds.domain_right_edge,
                data_file.ds.filter_bbox)
            ind += c.shape[0]
        return morton

    def _count_particles(self, data_file):
        return self.ds.parameters["particle_count"][data_file.file_id]



"""
Athena++ frontend tests



"""

#-----------------------------------------------------------------------------
# Copyright (c) 2013, yt Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
#-----------------------------------------------------------------------------

import numpy as np
from yt.testing import \
    assert_equal, \
    requires_file, \
    units_override_check, \
    assert_allclose
from yt.utilities.answer_testing.framework import \
    requires_ds, \
    small_patch_amr, \
    data_dir_load, \
    GenericArrayTest
from yt.frontends.athena_pp.api import AthenaPPDataset
from yt.convenience import load

_fields_disk = ("density", "velocity_r")

disk = "KeplerianDisk/disk.out1.00000.athdf"
@requires_ds(disk)
def test_disk():
    ds = data_dir_load(disk)
    assert_equal(str(ds), "disk.out1.00000")
    dd = ds.all_data()
    vol = (ds.domain_right_edge[0]**3-ds.domain_left_edge[0]**3)/3.0
    vol *= np.cos(ds.domain_left_edge[1])-np.cos(ds.domain_right_edge[1])
    vol *= ds.domain_right_edge[2].v-ds.domain_left_edge[2].v
    assert_allclose(dd.quantities.total_quantity("cell_volume"), vol)
    for field in _fields_disk:
        def field_func(name):
            return dd[field]
        yield GenericArrayTest(ds, field_func, args=[field])

_fields_AM06 = ("temperature", "density", "velocity_magnitude", "magnetic_field_x")

AM06 = "AM06/AM06.out1.00400.athdf"
@requires_ds(AM06)
def test_AM06():
    ds = data_dir_load(AM06)
    assert_equal(str(ds), "AM06.out1.00400")
    for test in small_patch_amr(ds, _fields_AM06):
        test_AM06.__name__ = test.description
        yield test

uo_AM06 = {
    'length_unit': (1.0, 'kpc'),
    'mass_unit': (1.0, 'Msun'),
    'time_unit': (1.0, 'Myr'),
}

@requires_file(AM06)
def test_AM06_override():
    # verify that overriding units causes derived unit values to be updated.
    # see issue #1259
    ds = load(AM06, units_override=uo_AM06)
    assert_equal(float(ds.magnetic_unit.in_units('gauss')),
                 9.01735778342523e-08)

@requires_file(AM06)
def test_units_override():
    units_override_check(AM06)

@requires_file(AM06)
def test_AthenaPPDataset():
    assert isinstance(data_dir_load(AM06), AthenaPPDataset)
